{/*
  Ported from timer.c by TRSOFT '97
 
  Implements small delays using the PC CTC.  The information needed
  to write this stuff came from Kris Heidenstrom's PC timing FAQ.

  THIS SOFTWARE IS PROVIDED AS IS AND WITHOUT WARRANTY OF ANY KIND,
  EITHER EXPRESSED OR IMPLIED.

}

unit Time;

interface

	procedure Setup_Timer(cmd:word);assembler;stdcall;
  	procedure Ms_Delay(ticks:word);
	procedure Us_Delay(ticks:word);
	procedure Reset_Timer;assembler;stdcall;


implementation

const
     TIMER  = $040;                  { base address of CTC }
     PORTB  = $061;                  { speaker control port }

 {#define TICKS(us)       ((unsigned) (1.193182*(us)+1))  /* us to ticks */}


{ Setup_Timer(cmd)

    Initialise PORTB and CTC.

}
procedure Setup_Timer(cmd:word);assembler;stdcall;
asm
 	pushf
   	cli
   	in      al,PORTB
   	and     al,$FD         { disable speaker }
   	or      al,1           { enable timer 2 gate }
   	out     PORTB,al
   	mov     ax,cmd
   	out     (TIMER+3),al   { send command }
   	popf

end;


{  Ms_Delay(ticks)

     Returns after roughly ticks*0.8381 microseconds (ticks must
     be non-negative and less than 32768 therefore the maximum
     delay is about 27 milliseconds).  On exit, initialises timer
     2 ready for us_delay() or us_pulse().

}
procedure Ms_Delay(ticks:word);
var
   hi,lo:word;
begin
    if ticks = 0 then
	begin
   	     setup_timer($90);      { timer 2, mode 0, 1-byte reload }
	     exit;
	end;

 	hi := ticks shr 8;
 	lo := ticks and $FF;

 	setup_timer($B0);          { timer 2, mode 0, 2-byte reload }

    asm
    	pushf
    	cli
    	mov     ax,lo
    	out     (TIMER+2),al    { load lo-byte }
    	in      al,PORTB        { waste some time }
    	mov     ax,hi
    	out     (TIMER+2),al    { load hi-byte }
    	popf
    @@mschk:
    	mov     al,$0
    	out     (TIMER+3),al    { latch count }
    	in      al,(TIMER+2)    { read latch lo-byte }
    	in      al,PORTB        { waste some time }
    	in      al,(TIMER+2)    { read latch hi-byte }
    	shl     al,1
    	jnc     @@mschk         { wait until count wraps around  }
    end;

    setup_timer($90);           { revert to mode 0, 1-byte reload }

end;



{ Us_Delay(ticks)

  Returns after roughly ticks*0.8381 microseconds (ticks must
  be non-negative and less than 128 therefore the maximum
  delay is about 107 microseconds).  Can only be called after
  timer 2 has been initialised.  Slightly more accurate than
  ms_delay() for short delays on slow machines.  Note, runs with
  interrupts off.
}
procedure Us_Delay(ticks:word);

begin
    if ticks = 0 then exit;

    asm
        pushf
    	cli
    	mov     ax,ticks
    	out     (TIMER+2),al    { reload timer 2 }
    @@uschk:
    	mov     al,$80
    	out     (TIMER+3),al    { latch count }
    	in      al,(TIMER+2)    { read latch }
    	shl     al,1
    	jnc     @@uschk         { wait until count wraps around  }
    	popf
    end;
end;


 {
   Reset_Timer()

     Attempts to restore timer 2 to default state.

 }
procedure Reset_Timer;assembler;stdcall;

asm
	pushf
 	cli
 	mov     al,$B6         { mode 3, two-byte reload }
 	out     (TIMER+3),al   { send command }
 	in      al,PORTB
        and     al,$FC         { disable speaker }
 	out     PORTB,al
 	popf
end;

End.